#include "../../include/database.h"
#include "../../include/package.h"
#include "../../include/error.h"
#include "../../include/util.h"

#include <sqlite3.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

bool lm_database_package_add(lm_database_t *db, lm_pkg_t *pkg){
  if(NULL == db || NULL == pkg){
    lm_error_set(LM_ERR_ArgNULL);
    return false;
  }

  char depends[lm_package_depend_strlen(pkg)];
  sqlite3_stmt *insert = NULL;
  bool ret = false;

  if(sqlite3_prepare(db->sql, queries[QUERY_INSERT_PACKAGE], strlen(queries[QUERY_INSERT_PACKAGE]), &insert, NULL) != SQLITE_OK){
    pdebug(__func__, "(%s) failed to prepare statement: %s", db->path, sqlite3_errmsg(db->sql));
    lm_error_set(LM_ERR_DbSqlPrepareFail);
    goto end;
  }

  sqlite3_bind_text(insert, 1, pkg->name, strlen(pkg->name), SQLITE_STATIC);
  sqlite3_bind_text(insert, 2, pkg->desc, strlen(pkg->desc), SQLITE_STATIC);
  sqlite3_bind_text(insert, 3, pkg->version, strlen(pkg->version), SQLITE_STATIC);
  sqlite3_bind_int64(insert, 4, pkg->size);

  if(!lm_package_depend_tostr(pkg, depends)){
    pdebug(__func__, "(%s) failed to convert depends to string: %s", db->path, lm_strerror());
    goto end;
  }
  sqlite3_bind_text(insert, 5, depends, strlen(depends), SQLITE_STATIC);

  if(sqlite3_step(insert) != SQLITE_OK){
    pdebug(__func__, "(%s) failed to execute insert statement: %s", db->path, sqlite3_errmsg(db->sql));
    lm_error_set(LM_ERR_DbSqlInsertFail);
    goto end;
  }

  ret = true;
end:
  if(NULL != insert)
    sqlite3_finalize(insert);
  return ret;
}

bool lm_database_package_find(lm_database_t *db, lm_pkg_t *pkg){
  return true;
}
