#include "../../include/package.h"
#include "../../include/error.h"
#include "../../include/types.h"
#include "../../include/util.h"

#include <stdlib.h>
#include <string.h>

size_t lm_package_depend_count(lm_pkg_t *pkg){
  size_t index = 0;

  if(NULL == pkg->depends)
    return index;

  while(pkg->depends[index] != NULL)
    index++;

  return index;
}

bool lm_package_depend_add(lm_pkg_t *pkg, char *depend){
  if(NULL == depend){
    lm_error_set(LM_ERR_ArgNULL);
    return false;
  }

  if(!package_name_valid(depend)){
    lm_error_set(LM_ERR_PkgBadName);
    return false;
  }

  if(NULL == pkg->depends){
    pkg->depends = malloc(sizeof(char*)*2);
    pkg->depends[0] = strdup(depend);
    pkg->depends[1] = NULL;
    return true;
  }

  size_t count = lm_package_depend_count(pkg);
  pkg->depends = realloc(pkg->depends, sizeof(char*)*(count+2));
  pkg->depends[count++] = strdup(depend);
  pkg->depends[count] = NULL;
  return true;
}

size_t lm_package_depend_strlen(lm_pkg_t *pkg){
  size_t len = 1;

  if(NULL == pkg->depends)
    return len;

  for(int i = 0; NULL != pkg->depends[i]; i++)
    len += strlen(pkg->depends[i])+1;

  return len;
}

bool lm_package_depend_tostr(lm_pkg_t *pkg, char *buffer){
  if(NULL == buffer || NULL == pkg){
    lm_error_set(LM_ERR_ArgNULL);
    return false;
  }

  if(NULL == pkg->depends){
    buffer[0] = 0;
    return true;
  }

  for(int i = 0; NULL != pkg->depends[i]; i++){
    if(i == 0){
      sprintf(buffer, "%s", pkg->depends[i]);
      continue;
    }
    sprintf(buffer, "%s,%s", buffer, pkg->depends[i]);
  }

  return true;
}
